#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ammo.phantom
~~~~~~~~~~~~

Functions to generate raw HTTP request.
"""

import pprint
pp = pprint.PrettyPrinter(indent=4)
import io
import httplib


class HttpCompiler(object):
    version = '3.1'

    def __init__(self, method=None, headers=None):
        self.method = method
        self.headers = headers

    def build_raw(self, url, method=None, body=None, headers=None, tag=None):
        '''
        @see http://docs.python.org/2.7/library/httplib.html#httplib.HTTPConnection.request
        '''
        headers = headers or self.headers
        assert isinstance(headers, dict)
        host = headers.get('Host', None)
        assert host
        method = method or self.method
        assert method

        conn = httplib.HTTPConnection(host)
        bio = io.BytesIO()
        bio.sendall = bio.write
        conn.sock = bio
        conn.request(method, url, body=body, headers=headers)
        return bio.getvalue()

    def build_phantom(self, *args, **kwargs):
        """Compile HTTP request in Yandex-tank(ammo) format.
        For more info about format @see
        http://phantom-doc-ru.readthedocs.org/en/latest/writing-a-phantom-conf.html#ammo-stpd
        All args and kwargs proxied to httplib.HTTPConnection.request method.

        Returns:
            str, ready to use with yandex-tank tool ammo file.
        """
        tag = kwargs.get('tag', None)
        req_ph = self.build_raw(*args, **kwargs)

        if kwargs.get('body'):  # httplib miss this for bodyless requests
            req_ph += '\r\n\r\n'

        if tag:
            req_ph = '{} {}\n'.format(len(req_ph), tag) + req_ph
        else:
            req_ph = '{}\n'.format(len(req_ph)) + req_ph

        req_ph += '\n'
        return req_ph



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
ammo.lunapark.generators
~~~~~~~~~~~~~~~~~~

This module provides helper functions for Lunapark ammo creation
"""

import sys
import urllib

# constraints
EOL = '\r\n'

# multi_part params
in_file = '/path/to/some/big/file'
content_type = 'multipart/form-data'
boundary = '------------------------------d7c3d7c4089e'
body_part = 'Content-Disposition: form-data; name="request"; filename="nm.xml"'
body_type = 'Content-Type: application/xml'


def make_post_req(req_path, body, context_up=None):
    ''' Create POST HTTP requset, calc size of it.
    Args:
        req_path: str - request path
        body: str - request body
        context_up: dict - update some request attributes if needed

    Returns:
        str - HTTP request in plain-text with size in bytes'''

    context = {
            'protocol': 'HTTP/1.1',
            'Host': 'target.changeme.test',
            'User-Agent': 'TankKG v.default',
            'Content-Type': 'application/x-www-form-urlencoded',
            'Referer': 'https://github.com/greggyNapalm/ammo',
            'Accept': '*/*',
            'Cookies': [],
        }

    if context_up:
        context.update(context_up)

    if 'session_id' in context:
        context['Cookies'].append('Session_id=' + context['session_id'])

    req = 'POST ' + req_path + ' ' + context['protocol'] + EOL
    req += 'Host: ' + context['Host'] + EOL
    req += 'User-Agent: ' + context['User-Agent'] + EOL
    req += 'Content-Type: ' + context['Content-Type'] + EOL
    req += 'Referer: ' + context['Referer'] + EOL
    req += 'Accept: ' + context['Accept'] + EOL
    req += 'Content-Length: ' + str(len(body)) + EOL
    req += EOL + body + EOL
    req += EOL
    return str(len(req)) + '\n' + req,


def make_multi_part(req_path, body):
    ''' Create multipart POST HTTP requset.
    Args:
        req_path: str - request path
        body: str - request body

    Returns:
        str - HTTP request in plain-text with size in bytes'''

    d_host = 'target.changeme.test'
    d_usr_agent = 'TankKG v.default'

    fmt_body = '--' + boundary + EOL
    fmt_body += body_part + EOL
    fmt_body += body_type + EOL + EOL
    fmt_body += body + EOL + EOL
    fmt_body += '--' + boundary + '--'

    req = 'POST ' + req_path + ' HTTP/1.1' + EOL
    req += 'Host: ' + d_host + EOL
    req += 'User-Agent: ' + d_usr_agent + EOL
    req += 'Accept: */*' + EOL
    req += 'Expect: 100-continue' + EOL
    req += 'Content-Type: multipart/form-data; boundary=' + boundary + EOL
    req += 'Content-Length: ' + str(len(fmt_body)) + EOL
    req += EOL + fmt_body + EOL + EOL
    return str(len(req)) + "\n" + req


def make_get_req(req_path, context_up=None):
    ''' Create GET HTTP requset, calc size of it.
    Args:
        req_path: str - request path
        context_up: dict - update some request attributes if needed

    Returns:
        str - HTTP request in plain-text with size in bytes'''

    context = {
            'protocol': 'HTTP/1.1',
            'Host': 'target.changeme.com',
            'User-Agent': 'TankKG v.default',
            'Referer': 'https://github.com/greggyNapalm/ammo',
            'Accept': '*/*',
            'Cookies': [],
        }

    if context_up:
        context.update(context_up)

    req = 'GET ' + req_path + ' ' + context['protocol'] + EOL
    req += 'Host: ' + context['Host'] + EOL
    req += 'User-Agent: ' + context['User-Agent'] + EOL
    req += 'Referer: ' + context['Referer'] + EOL
    req += 'Accept: ' + context['Accept'] + EOL

    if 'session_id' in context:
        context['Cookies'].append('Session_id=' + context['session_id'])

    if 'Connection' in context:
        req += 'Connection: ' + context['Connection'] + EOL

    req += EOL
    return str(len(req)) + '\n' + req

if __name__ == '__main__':
    try:
        body_file = open(in_file, "r")
        f_body = body_file.read()
        body_file.close()
    except IOError, e:
        print 'Cant\'t read ', in_file, '\n', e
        sys.exit(1)
    pass
    make_multi_part('/chnageme/api/', f_body)
    print '0',



# -*- coding: utf-8 -*-

import os

version_info = (0, 0, 7)
__version__ = ".".join(map(str, version_info))
__path = os.path.dirname(__file__)


class Mixer(object):
    '''Takes data chunks(requests) from sources, mix data in given proportion
    and yields chunks.
    Attributes:
        num_req: int, total requests needed. Not used if one of generators
        stops earlier.
    '''
    def __init__(self, num_req=10 ** 7):
        self.num_req = num_req
        self.gens = {}

    def add_generator(self, name, frequency, miner):
        '''Add one more data source to mix in.
        Args:
            name: str, name of data source(Helps to debug).
            frequency: float, percentil of source in result stream
            miner: generator obj, yelds requests.

        Returns:
            generator object, yield pids.
        '''
        assert isinstance(name, basestring)
        assert isinstance(frequency, float)

        self.gens[name] = {
            'frq': frequency,
            'miner': miner,
        }

    def __iter__(self):
        '''Iterate over generator till one of them ends(raises StopIteration).
        Returns:
            yields requetst from original generators in given proportion.
        '''

        if len(self.gens) < 1:
            raise ValueError('Have no generators to mix.')

        state = {}
        cntr = 0
        max_freq = max(g['frq'] for g in self.gens.values())
        for g_name, g_val in self.gens.iteritems():
            state[g_name] = {
                'proficit': 0,
                'ratio': g_val['frq'] / max_freq,
            }

        while True:
            for g_name, ctx in state.iteritems():
                if cntr >= self.num_req:
                    raise StopIteration('{} achieved'.format(self.num_req))
                ctx['proficit'] += ctx['ratio']
                if ctx['proficit'] >= 1:
                    for i in xrange(int(ctx['proficit'])):
                        ctx['proficit'] -= 1
                        cntr += 1
                        yield self.gens[g_name]['miner'].next()

        for each in self.__dict__.keys():
            yield self.__getattribute__(each)



from setuptools import setup

from ammo import __version__

setup(
    name='ammo',
    version=__version__,
    author='Gregory Komissarov',
    author_email='gregory.komissarov@gmail.com',
    description='Input data generator for different load tools',
    license='BSD',
    url='https://github.com/greggyNapalm/ammo',
    keywords=['phantom', 'yandex-tank', 'ammo'],
    packages=[
        'ammo',
        'ammo.lunapark',
        'ammo.phantom'
    ],
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
)
